﻿
#region Using Directives

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Windows.Media.Imaging;
using System.Xml.Linq;

#endregion

namespace TuneDepot.Models
{

    /// <summary>
    /// Represents the model, which loads and manages the album data.
    /// </summary>
    public static class MusicAlbumsModel
    {

        #region Constructor

        /// <summary>
        /// The static constructor of the music album model, which loads the album data from an XML file
        /// and makes it available.
        /// </summary>
        static MusicAlbumsModel()
        {

            // Load the XML data from the server
            IsDownloadCompleted = false;
            LoadXml();

        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Contains the music album data for all albums.
        /// </summary>
        public static List<Album> AlbumData { get; private set; }

        /// <summary>
        /// Gets all genres of the albums available in the store.
        /// </summary>
        public static List<string> AlbumGenres
        {

            get
            {

                // Create a list and add a member, which represents all genres
                List<string> genres = new List<string>();
                genres.Add("Alle");

                // Go through all albums and collect all genre names
                foreach (Album album in AlbumData)
                {

                    if (!genres.Contains(album.Genre))
                        genres.Add(album.Genre);

                }

                return genres;

            }

        }

        /// <summary>
        /// Get all artists of the albums available in the store.
        /// </summary>
        public static List<string> AlbumArtists
        {

            get
            {

                // Create a list and add a member, which represents all artists
                List<string> artists = new List<string>();
                artists.Add("Alle");

                // Go through all albums and collect all artist names
                foreach (Album album in AlbumData)
                {

                    if (!artists.Contains(album.Artist))
                        artists.Add(album.Artist);

                }

                return artists;

            }

        }

        /// <summary>
        /// Gets the price of the most expensive album in the store.
        /// </summary>
        public static double AlbumMaximumPrice
        {

            get
            {

                // Calculate the maximum price
                double max = 0.0;

                foreach (Album album in AlbumData)
                {

                    double price = Convert.ToDouble(album.Price.Substring(0, album.Price.Length - 1));

                    if (price > max)
                        max = price;

                }

                return max;

            }

        }

        /// <summary>
        /// Gets the price of the most cheapest album in the store.
        /// </summary>
        public static double AlbumMinimumPrice
        {

            get
            {

                // Calculate the minimum price
                double min = Double.MaxValue;

                foreach (Album album in AlbumData)
                {

                    double price = Convert.ToDouble(album.Price.Substring(0, album.Price.Length - 1));

                    if (price < min)
                        min = price;

                }

                return min;

            }

        }

        /// <summary>
        /// Gets whether the download is completed or not.
        /// </summary>
        public static bool IsDownloadCompleted { get; private set; }

        #endregion

        #region Private Methods

        /// <summary>
        /// Loads the XML data file from the server.
        /// </summary>
        /// <returns>
        /// Return the parsed XML from the server.
        /// </returns>
        private static void LoadXml()
        {

            // Download the XML document
            WebClient loader = new WebClient();
            loader.DownloadStringCompleted += new DownloadStringCompletedEventHandler(loader_DownloadStringCompleted);
            loader.DownloadStringAsync(new Uri("http://localhost:40001/Data/AlbumData.xml", UriKind.Absolute));

        }

        /// <summary>
        /// Parses the XML data and returns it as a list.
        /// </summary>
        /// <param name="data">
        /// The XML document, which is to be parsed.
        /// </param>
        /// <returns>
        /// Returns a list with the album data.
        /// </returns>
        private static List<Album> ParseAlbumData(XDocument data)
        {

            // Parse the XML data
            IEnumerable<Album> albums = from album in data.Descendants("Album")
                                                 select new Album
                                                 {

                                                     Name = (string)album.Attribute("Name"),
                                                     Artist = (string)album.Attribute("Artist"),
                                                     Genre = (string)album.Attribute("Genre"),
                                                     Price = (string)album.Attribute("Price"),
                                                     ImagePath = (string)album.Attribute("Image")

                                                 };

            // Return the albums as an observable collection
            return new List<Album>(albums);

        }

        #endregion

        #region Event Handling

        /// <summary>
        /// Is called, when the XML data is downloaded from the server.
        /// </summary>
        /// <param name="sender">
        /// The sender, in this case the loader.
        /// </param>
        /// <param name="e">
        /// The event arguments, which contain the downloaded XML data.
        /// </param>
        static void loader_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {

            // Parse the XML data
            AlbumData = ParseAlbumData(XDocument.Parse(e.Result));
            IsDownloadCompleted = true;
            DownloadCompleted();

        }

        #endregion

        #region Events

        /// <summary>
        /// The event handler for the DownloadCompleted event.
        /// </summary>
        public delegate void DownloadCompletedEventHandler();

        /// <summary>
        /// The event, which is raised, when the data was completely downloaded from the server.
        /// </summary>
        public static event DownloadCompletedEventHandler DownloadCompleted;

        #endregion

    }

    /// <summary>
    /// Represents an album with all its information.
    /// </summary>
    public class Album
    {

        #region Public Properties

        /// <summary>
        /// Contains the name of the album.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Contains the name of the artist.
        /// </summary>
        public string Artist { get; set; }

        /// <summary>
        /// Gets or set the genre of the album.
        /// </summary>
        public string Genre { get; set; }

        /// <summary>
        /// Contains the price of the album.
        /// </summary>
        public string Price { get; set; }

        /// <summary>
        /// Contains the path to the album cover.
        /// </summary>
        private string imagePath;

        /// <summary>
        /// Contains the path to the album cover.
        /// </summary>
        public string ImagePath
        {

            get
            {

                return imagePath;

            }
            
            set
            {

                imagePath = value;
                Image = new BitmapImage(new Uri("http://localhost:40001/ClientBin/Images/" + imagePath));

            }
        
        }

        /// <summary>
        /// Gets the album cover as an bitmap image.
        /// </summary>
        public BitmapImage Image { get; private set; }

        #endregion

    }

}
