﻿
#region Using Directives

using System;
using TuneDepot.Models;
using System.ComponentModel;
using System.Collections.Generic;
using System.Windows;
using System.Collections.ObjectModel;
using System.Linq;

#endregion

namespace TuneDepot.ViewModels
{

    /// <summary>
    /// Represents the view model of the main page.
    /// </summary>
    public class MainPageViewModel : INotifyPropertyChanged
    {

        #region Constructor

        /// <summary>
        /// The constructor of the main page view model.
        /// </summary>
        public MainPageViewModel()
        {

            // Get the from the model
            Albums = MusicAlbumsModel.AlbumData;
            artists = new ObservableCollection<string>(MusicAlbumsModel.AlbumArtists);
            SelectedArtist = artists[0];
            genres = new ObservableCollection<string>(MusicAlbumsModel.AlbumGenres);
            SelectedGenre = genres[0];
            maximumPrice = MusicAlbumsModel.AlbumMaximumPrice;
            minimumPrice = MusicAlbumsModel.AlbumMinimumPrice;

            // Set the current maximum and the minimum price
            CurrentMaximumPrice = MaximumPrice;
            CurrentMinimumPrice = MinimumPrice;

        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Contains a list of all albums that match the filter options.
        /// </summary>
        private ObservableCollection<Album> filteredAlbums = new ObservableCollection<Album>();

        /// <summary>
        /// Gets a list of all albums that match the filter options.
        /// </summary>
        public ObservableCollection<Album> FilteredAlbums
        {

            get
            {

                // Return the list of filtered albums
                return filteredAlbums;

            }

        }

        /// <summary>
        /// Contains a list of all artists.
        /// </summary>
        private ObservableCollection<string> artists;

        /// <summary>
        /// Gets a list of all artists.
        /// </summary>
        public ObservableCollection<string> Artists
        {

            get
            {

                return artists;

            }

        }

        /// <summary>
        /// Contains a list of all genres.
        /// </summary>
        public ObservableCollection<string> genres;

        /// <summary>
        /// Gets a list of all genres.
        /// </summary>
        public ObservableCollection<string> Genres
        {

            get
            {

                return genres;

            }

        }

        /// <summary>
        /// Contains the maximum album price.
        /// </summary>
        private double maximumPrice;

        /// <summary>
        /// Gets the maximum album price.
        /// </summary>
        public double MaximumPrice
        {

            get
            {

                return maximumPrice;

            }

        }

        /// <summary>
        /// Contains the minimum album price.
        /// </summary>
        public double minimumPrice;

        /// <summary>
        /// Gets the minimum album price.
        /// </summary>
        public double MinimumPrice
        {

            get
            {

                return minimumPrice;

            }

        }

        /// <summary>
        /// Contains the current maximum price.
        /// </summary>
        private double currentMaximumPrice;

        /// <summary>
        /// Gets or sets the current maximum price.
        /// </summary>
        public double CurrentMaximumPrice
        {

            get
            {

                return currentMaximumPrice;

            }

            set
            {

                currentMaximumPrice = (int)(value * 100 + 0.5) / 100.0;
                NotifyPropertyChanged("CurrentMaximumPrice");
                this.FilterAlbums();

            }

        }

        /// <summary>
        /// Contains the current minimum price.
        /// </summary>
        private double currentMinimumPrice;

        /// <summary>
        /// Gets or sets the current minimum price.
        /// </summary>
        public double CurrentMinimumPrice
        {

            get
            {

                return currentMinimumPrice;

            }

            set
            {

                currentMinimumPrice = (int)(value * 100 - 0.5) / 100.0;
                NotifyPropertyChanged("CurrentMinimumPrice");
                this.FilterAlbums();

            }

        }

        /// <summary>
        /// Contains the selected artists.
        /// </summary>
        private string selectedArtist;

        /// <summary>
        /// Gets or set the selected artist.
        /// </summary>
        public string SelectedArtist
        {

            get
            {

                return selectedArtist;

            }

            set
            {

                selectedArtist = value;
                NotifyPropertyChanged("SelectedArtist");
                this.FilterAlbums();

            }

        }

        /// <summary>
        /// Contains the selected genre.
        /// </summary>
        private string selectedGenre;

        /// <summary>
        /// Gets or set the selected genre.
        /// </summary>
        public string SelectedGenre
        {

            get
            {

                return selectedGenre;

            }

            set
            {

                selectedGenre = value;
                NotifyPropertyChanged("SelectedGenre");
                this.FilterAlbums();

            }

        }

        /// <summary>
        /// Contains the currently selected album.
        /// </summary>
        private Album selectedAlbum;

        /// <summary>
        /// Gets or sets the currently selected album.
        /// </summary>
        public Album SelectedAlbum
        {

            get
            {

                return selectedAlbum;

            }

            set
            {

                selectedAlbum = value;
                NotifyPropertyChanged("SelectedAlbum");

            }

        }

        /// <summary>
        /// Contains the content of the shopping cart.
        /// </summary>
        private ObservableCollection<Album> shoppingCart = new ObservableCollection<Album>();

        /// <summary>
        /// Gets or sets the content of the shopping cart.
        /// </summary>
        public ObservableCollection<Album> ShoppingCart
        {

            get
            {

                return shoppingCart;

            }

            set
            {

                shoppingCart = value;
                NotifyPropertyChanged("ShoppingCart");

            }

        }

        /// <summary>
        /// Contains the currently selected shopping cart item.
        /// </summary>
        private Album selectedShoppingCartItem;

        /// <summary>
        /// Gets or sets the currently selected shopping cart item.
        /// </summary>
        public Album SelectedShoppingCartItem
        {

            get
            {

                return selectedShoppingCartItem;

            }

            set
            {

                selectedShoppingCartItem = value;
                NotifyPropertyChanged("SelectedShoppingCartItem");

            }

        }

        #endregion

        #region Private Properties

        /// <summary>
        /// Contains the album data.
        /// </summary>
        private List<Album> Albums = new List<Album>();

        #endregion

        #region Commands

        /// <summary>
        /// Gets a command, which adds the currently selected album to the shopping cart.
        /// </summary>
        public RelayCommand AddToShoppingCart
        {

            get
            {

                return new RelayCommand(
                    delegate(object parameter)
                    {

                        if (SelectedAlbum == null)
                        {

                            MessageBox.Show("Sie haben kein Album ausgewählt.", "Kein Album ausgewählt", MessageBoxButton.OK);

                        }
                        else
                        {

                            ShoppingCart.Add(SelectedAlbum);

                        }

                    });

            }

        }

        /// <summary>
        /// Gets a command, which deletes an item, from the shopping cart.
        /// </summary>
        public RelayCommand DeleteFromShoppingCart
        {

            get
            {

                return new RelayCommand(
                    delegate(object parameter)
                    {

                        if (SelectedShoppingCartItem == null)
                        {

                            MessageBox.Show("Sie haben kein Album ausgewählt.", "Kein Album ausgewählt", MessageBoxButton.OK);

                        }
                        else
                        {

                            ShoppingCart.Remove(SelectedShoppingCartItem);

                        }

                    });

            }

        }

        /// <summary>
        /// Gets a command, which buys the items from the shopping cart.
        /// </summary>
        public RelayCommand BuyShoppingCarItems
        {

            get
            {

                return new RelayCommand(
                    delegate(object parameter)
                    {

                        if (ShoppingCart.Count == 0)
                        {

                            MessageBox.Show("Sie haben kein Album/keine Alben im Einkaufswagen.",
                                "Einkaufswagen leer", MessageBoxButton.OK);

                        }
                        else
                        {

                            string message;
                            if (ShoppingCart.Count == 1)
                                message = "Sie haben das Album gekauft";
                            else
                                message = "Sie haben die Alben gekauft";

                            MessageBox.Show(message, "Gekauft", MessageBoxButton.OK);

                            ShoppingCart.Clear();

                        }

                    });

            }

        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Raises a property changed event.
        /// </summary>
        /// <param name="propertyName">
        /// The name of the property that has changed.
        /// </param>
        private void NotifyPropertyChanged(string propertyName)
        {

            // Raise the property changed event
            PropertyChangedEventArgs e = new PropertyChangedEventArgs(propertyName);

            // Check if anybody is listening to the property changed event, if so, raise it
            if (PropertyChanged != null)
                PropertyChanged(this, e);

        }

        /// <summary>
        /// Filters the albums according to the currently set filter options.
        /// </summary>
        private void FilterAlbums()
        {

            // First of all delete all albums from the previous list
            filteredAlbums.Clear();

            // Filter the Album list and put them into a new list
            IEnumerable<Album> albums = from album in Albums
                                        where (album.Artist == SelectedArtist || SelectedArtist == "Alle") &&
                                              (album.Genre == SelectedGenre  || SelectedGenre == "Alle") &&
                                              Convert.ToDouble(album.Price.Substring(0, album.Price.Length - 1)) >= CurrentMinimumPrice &&
                                              Convert.ToDouble(album.Price.Substring(0, album.Price.Length - 1)) <= CurrentMaximumPrice
                                        select new Album
                                        {

                                            Name = album.Name,
                                            Artist = album.Artist,
                                            Genre = album.Genre,
                                            Price = album.Price,
                                            ImagePath = album.ImagePath

                                        };

            // Copy the list into the filtered Albums list
            foreach (Album album in albums) filteredAlbums.Add(album);

        }

        #endregion

        #region Events

        /// <summary>
        /// Is raised, when a property is changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

    }

}
